'From Squeak 2.0 of May 22, 1998 on 26 May 1998 at 5:36:59 pm'!
ReferenceStream subclass: #SmartRefStream
	instanceVariableNames: 'writing structures steady reshaped renamed superclasses progressBar objCount classInstVars '
	classVariableNames: 'ScannedObject '
	poolDictionaries: ''
	category: 'System-Object Storage'!

!Object methodsFor: 'copying' stamp: 'tk 5/22/1998 12:16'!
veryDeepCopy
	"Do a complete tree copy using a dictionary.  An object in the tree twice is only copied once.  Both pointers point to one new copy.  Uses ReferenceStream.  To see where the copying stops, see DataStream.typeIDFor: and implementors of objectToStoreOnDataStream"

	| dummy refs class index sub val hashers new |
	dummy _ ReferenceStream on: (DummyStream on: nil).
		"Write to a fake Stream, not a file"
	"Collect all objects"
	dummy rootObject: self.	"inform him about the root"
	dummy nextPut: self.	"Do the traverse of the tree"
	refs _ dummy references.	"all the objects"
	"For each object, make a simple copy, then replace all fields with new copy from dict"
	refs keysDo: [:each | refs at: each put: each shallowCopy].
		"Watch out for classes that do extra things in copy but not in shallowCopy"
	hashers _ OrderedCollection new.
	refs associationsDo: [:assoc | 
		assoc key == assoc value ifFalse: ["is a new object"
			new _ assoc value.
			class _ new class.
			class isVariable
				ifTrue: 
					[index _ new basicSize.
					[index > 0] whileTrue: 
						[sub _ new basicAt: index.
						(val _ refs at: sub ifAbsent: [nil]) ifNotNil: [
								"If not in refs, then the right value is already in the field"
								new basicAt: index put: val].
						index _ index - 1]].
			index _ class instSize.
			[index > 0] whileTrue: 
				[sub _ new instVarAt: index.
				(val _ refs at: sub ifAbsent: [nil]) ifNotNil: [
						"If not in refs, then the right value is already in the field"
						new instVarAt: index put: val].
				index _ index - 1].
			(new respondsTo: #rehash) ifTrue: [hashers add: new].
			]].
	"Force new sets and dictionaries to rehash"
	hashers do: [:each | each rehash].

	^ refs at: self! !


!Morph methodsFor: 'copying' stamp: 'tk 5/22/1998 12:17'!
veryDeepCopy
	"Do a complete tree copy using a dictionary.  An object in the tree twice is only copied once.  Both pointers point to one new copy.  Uses ReferenceStream.  To see where the copying stops, see DataStream.typeIDFor: and implementors of objectToStoreOnDataStream"

	self allMorphsDo: [:m | m prepareToBeSaved].		"Actors fix open scriptEditors"
	^ super veryDeepCopy ! !

!Morph methodsFor: 'copying' stamp: 'tk 5/22/1998 12:17'!
veryDeepCopyWithoutCostumee

	| hold copy holdState |
	hold _ costumee.
	holdState _ self valueOfProperty: #actorState.
	holdState ifNotNil: [self removeProperty: #actorState].
	costumee _ nil.
	copy _ self veryDeepCopy.
	holdState ifNotNil:
		[copy setProperty: #actorState toValue: (holdState copyWithPlayerReferenceNilled)].
	costumee _ hold.
	holdState ifNotNil: [self setProperty: #actorState toValue: holdState].
	^ copy! !

!Morph methodsFor: 'other' stamp: 'tk 5/26/1998 15:33'!
saveOnFile
	"Ask the user for a filename and save myself on a SmartReferenceStream file.  Writes out the version and class structure.  The file is fileIn-able.  UniClasses will be filed out."

	| aFileName fileStream |
	aFileName _ ('my ', self class name) asFileName.	"do better?"
	aFileName _ FillInTheBlank request: 'File name? (".morph" will be added to end)' 
			initialAnswer: aFileName.
	aFileName size == 0 ifTrue: [^ self beep].
	self allMorphsDo: [:m | m prepareToBeSaved].

	fileStream _ FileStream newFileNamed: aFileName,'.morph'.
	fileStream fileOutClass: nil andObject: self.	"Puts UniClass definitions out anyway"! !


!ReferenceStream class methodsFor: 'all' stamp: 'tk 5/26/1998 14:51'!
versionCode
    "Answer a number representing the 'version' of the ReferenceStream facility; this is stashed at the beginning of ReferenceStreams, as a secondary versioning mechanism (the primary one is the fileTypeCode).   At present, it serves for information only, and is not checked for compatibility at reload time, but could in future be used to branch to variant code. "

	" 1 = original version 1992"
	" 2 = HyperSqueak.  PathFromHome used for Objs outside the tree.  SqueakSupport SysLibrary for shared globals like Display and StrikeFonts.  File has version number, class structure, then an IncomingObjects manager.  8/16/96 tk.  
	Extended to SmartRefStream.  class structure also keeps superclasse chain.  Does analysis on structure to see when translation methods are needed.  Embedable in file-ins.  (factored out HyperSqueak support)  Feb-May 97 tk"
	" 3 = Reference objects are byte offsets relative to the start of the object portion of the file.  Rectangles with values -2048 to 2047 are encoded compactly."
	" 4 = If UniClasses (subclasses of Player) have class instance variables, append their values in the form (#Class43 (val1 val2 vla3)).  An array of those.  Can still read version 3."
	^ 4! !


!SmartRefStream commentStamp: 'tk 5/26/1998 17:37' prior: 0!
Ordinary ReferenceStreams cannot bring in objects whose instance variables have changed.  This class does that.  1/13/97 tk

>>>>See SmartRefStream.aComment for details of how to transform incoming classes to newer versions.<<<< 

* Allows incoming object to have fewer instance variables than the current class.
* Recognises that conversion to new versions is only done after the fact.  Nothing special needs to be done at object file write time.  
* Someday will prompt for a new version number when there are new inst vars with same initials, and tells how to make conversion work when some files have already been written.
* Works best with only one (large) tree of objects per file.  Can nextPut: more than once, but each gets its own class structure description, which is big.  Writes triplets of (version, class structure array, object) on the file.
* Version of a class is indicated by the first letters of all instance varaibles followed by a class version number.  Form has inst vars "bits width height depth offset", so version 2 of it has version tag #bwhdo2.
* Methods to convert versions of objects are named 
	Form new convertbwhdo2: aDictionary bwhdo3: aSmartRefStream.
	Note that aDictionary has (old inst var name -> value) 
* Prompts the user to write (or file in) a conversion method when needed.
* (Does not use DiskProxy, DiskProxyQ, or IOWeakArray)
* Allows a change of instance vars in a superclass to be handles by just one conversion method.  Figures out what to do to the subclasses.
* The file can start out in the normal fileOut format.  Definitions of new classes on the front and an object at the end.

writing		true if writing a file.  To avoid mixing next with nextPut:.
structures 	Dictionary of (#Rectangle -> #(<classVersionInteger> origin corner)).  Inst 
				var names are strings.
steady 		Set of Classes who have the same structure now as on the incoming file.
				Includes classes with same inst vars except some added on end now.
reshaped 	Dictionary of Classes who have a different structure now as on the incoming file.  
				Includes those with same inst vars but new version number.
				(old class name -> method selector to fill in data for version to version)
renamed	Dictionary of Classes who have a different name. 
				(old class name symbol -> new class name).  Then look new class up in reshaped.
topCall		Tells if next or nextPut: coming from the outside.  nil if outside, a Context if 
				internal call.

>>>>> See DataStream.typeIDFor: for where the tangle of objects is clipped, so the whole system will not be written on the file.

UniClasses are classes for the instance specific behavior of just one instance.  Subclasses of Player are an example.  When a UniClass is read in, and a class of the same name already exists, the incoming one is renamed.  ObjectScanner converts the filed in code.

Values in instance variables of UniClasses are stored in the array that tells the class structure.  It is the second of the three top level objects.  Current format is:  #('class structure' structures 'superclasses' superclasses 'class inst vars' classInstVars).
 !

!SmartRefStream methodsFor: 'all' stamp: 'tk 5/26/1998 14:47'!
instVarInfo: anObject
	"Return the object to write on the outgoing file that contains the structure of each class we are about to write out.  Must be an Array whose first element is 'class structure'.  Its second element is a Dictionary of pairs of the form #Rectangle -> #(<classVersion> 'origin' 'corner').  "

	"Make a pass through the objects, not writing, but recording the classes.  Construct a database of their inst vars and any version info (classVersion)."

	| dummy refs cls newSupers |
	dummy _ ReferenceStream on: (DummyStream on: nil).
		"Write to a fake Stream, not a file"
	"Collect all objects"
	dummy rootObject: anObject.	"inform him about the root"
	dummy nextPut: anObject.
	refs _ dummy references.
	self uniClassInstVarsRefs: dummy.	"catalog the extra objects in UniClass inst vars"
	structures _ Dictionary new.
	superclasses _ Dictionary new.
	objCount _ refs size.		"for progress bar"
		"Note that Dictionary must not change its implementation!!  If it does, how do we read this reading information?"
	refs keysDo: [:each | 
		cls _ each class.
		cls class == Metaclass ifFalse: [
			structures at: cls name put: false]].
	"Save work by only computing inst vars once for each class"
	newSupers _ Set new.
	structures keysDo: [:nm | 
		cls _ Smalltalk at: nm.
		cls allSuperclasses do: [:aSuper |
			structures at: aSuper name ifAbsent: [newSupers add: aSuper name]]].
			"Don't modify structures during iteration"
	newSupers do: [:nm | structures at: nm put: 3].	"Get all superclasses into list"
	structures keysDo: [:nm | "Nothing added to classes during loop"
		cls _ Smalltalk at: nm.
		structures at: nm put: 
			((Array with: cls classVersion), (cls allInstVarNames)).
		superclasses at: nm ifAbsent: [
				superclasses at: nm put: cls superclass name]].
	self saveClassInstVars.	"of UniClassses"
	^ (Array with: 'class structure' with: structures with: 'superclasses' with: superclasses)! !

!SmartRefStream methodsFor: 'all' stamp: 'tk 5/26/1998 15:09'!
moreObjects
	"Return true if there appears to be another object following this one on the file."

	| byte |
	byteStream atEnd ifTrue: [^ false].	"off end of file"
	(byte _ byteStream peek) ifNil: [^ false].	"off end of file"
	byte = 33 "$!! asciiValue" ifTrue: [^ false].
	byte = 0 ifTrue: [^ false].
	^ byte <= RefTypes size		"between 1 and 16"! !

!SmartRefStream methodsFor: 'all' stamp: 'tk 5/26/1998 14:55'!
next
	"Really write three objects: (version, class structure, object). But only when called from the outside.  "

	| version ss object |
	^ topCall == nil 
		ifTrue: 
			[topCall _ thisContext.
			writing _ false.
			version _ super next.
			version class == SmallInteger ifFalse: [^ version].	
				"version number, else just a regular object, not in our format, "
			ss _ super next.
			ss class == Array ifFalse: [^ ss].  "just a regualr object"
			(ss at: 1) = 'class structure' ifFalse: [^ ss].
			structures _ ss at: 2.
			superclasses _ (ss size > 3 and: [(ss at: 3) = 'superclasses']) 
				ifTrue: [ss at: 4]		"class name -> superclass name"
				ifFalse: [Dictionary new].
			(self verifyStructure = 'conversion method needed') ifTrue: [^ nil].
			writing _ #Unclassed.	"Pass 1"
			object _ super next.	"all the action here"
			self restoreClassInstVars.		"for UniClasses. version 4"

			topCall _ writing _ nil.	"reset it"
			object]
		ifFalse:
			[super next]
! !

!SmartRefStream methodsFor: 'all' stamp: 'tk 5/26/1998 14:46'!
nextPut: anObject
	"Really write three objects: (version, class structure, object).  But only when called from the outside.  If any instance-specific classes are present, prepend their source code.  byteStream will be in fileOut format.
	You can see an analysis of which objects are written out by doing: 
	(SmartRefStream statsOfSubObjects: anObject)
	(SmartRefStream tallyOfSubObjects: anObject)
	(SmartRefStream subObjects: anObject ofClass: aClass)"

| info |
topCall == nil 
	ifTrue:
		[topCall _ anObject.  writing _ true. 
		'Please wait while objects are counted' 
			displayProgressAt: Sensor cursorPoint
			from: 0 to: 10
			during: [:bar | info _ self instVarInfo: anObject].
		self appendClassDefns.	"For instance-specific classes"
		'Writing an object file' displayProgressAt: Sensor cursorPoint
			from: 0 to: objCount*4	"estimate"
			during: [:bar |
				objCount _ 0.
				progressBar _ bar.
				self setStream: byteStream.	"set basePos, but keep any class renames"
				super nextPut: ReferenceStream versionCode.
				super nextPut: info.
				super nextPut: anObject.		"<- the real writing"
				classInstVars size > 0 ifTrue: [super nextPut: classInstVars]].
					"Note: the terminator, $!!, is not doubled inside object data"
		"references is an IDict of every object that got written"
		byteStream ascii.
		byteStream nextPutAll: '!!'; cr; cr.
		topCall _ writing _ progressBar _ nil]	"reset it"
	ifFalse:
		[super nextPut: anObject.
		progressBar ifNotNil: [progressBar value: (objCount _ objCount + 1)]].
! !

!SmartRefStream methodsFor: 'all' stamp: 'tk 5/26/1998 15:20'!
restoreClassInstVars
	"Install the values of the class instance variables of UniClasses (i.e. scripts slotInfo costumeDictionary).  Later: Watch for changed structure."

	| normal aName newName newCls |
	self moreObjects ifFalse: [^ self]. 	"are no UniClasses with class inst vars" 
	classInstVars _ super next.	"Array of arrays"
	normal _ Object class instSize.	"might give trouble if Player class superclass changes size"
	classInstVars do: [:list |
		aName _ (list at: 1) asSymbol.
		newName _ renamed at: aName ifAbsent: [aName].
		newCls _ Smalltalk at: newName 
				ifAbsent: [self error: 'UniClass definition missing'].
		newCls class instSize = (normal+(list size)-1) ifFalse: [
			self error: 'UniClass superclass class has changed size'].
			"Need to install a conversion method mechanism"
		2 to: list size do: [:ii |
			newCls instVarAt: normal+ii-1 put: (list at: ii)]].
! !

!SmartRefStream methodsFor: 'all' stamp: 'tk 5/26/1998 15:28'!
saveClassInstVars
	"Install the values of the instance variables of UniClasses.  classInstVars is an array of arrays (#Player3 (Player3 class's inst var scripts) (Player3 class's inst var slotInfo) ...) "

	| normal mySize list |
	classInstVars _ OrderedCollection new: 100.
	normal _ Object class instSize.
	self uniClasesDo: [:aUniClass | 
		mySize _ aUniClass class instSize.
		mySize = normal ifFalse: [
			list _ OrderedCollection new.
			list add: aUniClass name.	"a symbol"
			normal+1 to: mySize do: [:ii | 
				list addLast: (aUniClass instVarAt: ii)]].
		classInstVars add: list asArray].
	classInstVars _ classInstVars asArray.
	! !

!SmartRefStream methodsFor: 'all' stamp: 'tk 5/26/1998 15:32'!
uniClassInstVarsRefs: dummy
	"If some of the objects seen so far are instances UniClasses, check the UniClasses for extra class inst vars, and send them to the steam also.  The new objects get added to (dummy references), where they will be noticed by the caller.  They will wind up in the structures array and will be written on the disk by class." 

| uniClasses normal more aUniClass mySize |
"Any classes used in the structure of classInstVars must be written out also!!
	dummy nextPut: (OrderedCollection new).
But for now, only uses Arrays."

uniClasses _ Set new.
normal _ Object class instSize.
more _ true.
[more] whileTrue: [
	more _ false.
	dummy references keysDo: [:each | "any faster way to do this?"
		(aUniClass _ each class) isSystemDefined ifFalse: [
			(uniClasses includes: aUniClass name) ifFalse: [
				mySize _ aUniClass class instSize.
				normal+1 to: mySize do: [:ii | 
					more _ true.
					dummy nextPut: (aUniClass instVarAt: ii)].
				uniClasses add: aUniClass name]]]].
"References dictionary is modified as the loop proceeds, but we will catch any we missed on the next cycle."

! !


