'From Squeak 2.0 of May 22, 1998 on 28 May 1998 at 12:16:44 pm'!
"Change Set:		FourEdits
Date:			28 May 1998
Author:			Dan Ingalls

Four handy edit commands...
Ctl-U	Convert linefeeds to carriage returns in selection - useful for some FTP sites
Ctl-X	Force selection to lowercase
Ctl-Y	Force selection to uppercase
Ctl-Z	Capitalize all words in selection
"!


!ParagraphEditor methodsFor: 'editing keys' stamp: 'di 5/28/1998 11:58'!
changeLfToCr: characterStream 
	"Replace all LFs by CRs.
	Triggered by Cmd-U -- useful when getting code from FTP sites"
	| cr lf |
	sensor keyboard.		"flush the triggering cmd-key character"
	cr _ Character cr.  lf _ Character linefeed.
	self replaceSelectionWith: (Text fromString:
			(self selection string collect: [:c | c = lf ifTrue: [cr] ifFalse: [c]])).
	^ true! !

!ParagraphEditor methodsFor: 'editing keys' stamp: 'di 5/28/1998 12:06'!
makeCapitalized: characterStream 
	"Force the current selection to uppercase.  Triggered by Cmd-X."
	| prev |
	sensor keyboard.		"flush the triggering cmd-key character"
	prev _ $-.  "not a letter"
	self replaceSelectionWith: (Text fromString:
			(self selection string collect:
				[:c | prev _ prev isLetter ifTrue: [c asLowercase] ifFalse: [c asUppercase]])).
	^ true! !

!ParagraphEditor methodsFor: 'editing keys' stamp: 'di 5/28/1998 12:00'!
makeLowercase: characterStream 
	"Force the current selection to lowercase.  Triggered by Cmd-X."

	sensor keyboard.		"flush the triggering cmd-key character"
	self replaceSelectionWith: (Text fromString: (self selection string asLowercase)).
	^ true! !

!ParagraphEditor methodsFor: 'editing keys' stamp: 'di 5/28/1998 12:00'!
makeUppercase: characterStream 
	"Force the current selection to uppercase.  Triggered by Cmd-X."

	sensor keyboard.		"flush the triggering cmd-key character"
	self replaceSelectionWith: (Text fromString: (self selection string asUppercase)).
	^ true! !


!ParagraphEditor class methodsFor: 'keyboard shortcut tables' stamp: 'di 5/28/1998 11:31'!
initializeShiftCmdKeyShortcuts
	"Initialize the shift-command-key (or control-key) shortcut table."
	"NOTE: if you don't know what your keyboard generates, use Sensor kbdTest"

	| cmdMap cmds |
	"shift-command and control shortcuts"
	cmdMap _ Array new: 256.  "use temp in case of a crash"
	cmdMap atAllPut: #noop:.
	cmdMap at: ( 1 + 1) put: #cursorHome:.			"home key"
	cmdMap at: ( 4 + 1) put: #cursorEnd:.			"end key"
	cmdMap at: ( 8 + 1) put: #forwardDelete:.			"ctrl-H or delete key"
	cmdMap at: (13 + 1) put: #crWithIndent:.			"ctrl-Return"
	cmdMap at: (27 + 1) put: #selectCurrentTypeIn:.	"escape key"
	cmdMap at: (28 + 1) put: #cursorLeft:.			"left arrow key"
	cmdMap at: (29 + 1) put: #cursorRight:.			"right arrow key"
	cmdMap at: (30 + 1) put: #cursorUp:.				"up arrow key"
	cmdMap at: (31 + 1) put: #cursorDown:.			"down arrow key"
	cmdMap at: (45 + 1) put: #changeEmphasis:.		"cmd-sh-minus"
	cmdMap at: (61 + 1) put: #changeEmphasis:.		"cmd-sh-plus"
	cmdMap at: (127 + 1) put: #forwardDelete:.		"del key"

	"Note: Command key overrides shift key, so, for example, cmd-shift-9 produces $9 not $("
	'9[,''' do: [ :char | cmdMap at: (char asciiValue + 1) put: #shiftEnclose: ].	"({< and double-quote"
	"Note: Must use cmd-9 or ctrl-9 to get '()' since cmd-shift-9 is a Mac FKey command."
	cmdMap at: (27 + 1) put: #shiftEnclose:.	"ctrl-["

	cmds _ #(
		$a	argAdvance:
		$b	browseItHere:
		$c	compareToClipboard:
		$d	duplicate:
		$e	methodStringsContainingIt:
		$f	displayIfFalse:
		$j	doAgainMany:
		$k	changeStyle:
		$n	referencesToIt:
		$r	indent:
		$l	outdent:
		$s	search:
		$t	displayIfTrue:
		$u	changeLfToCr:
		$v	pasteInitials:
		$w	methodNamesContainingIt:
		$x	makeLowercase:
		$y	makeUppercase:
		$z	makeCapitalized:
	).
	1 to: cmds size by: 2 do: [ :i |
		cmdMap at: ((cmds at: i) asciiValue + 1)			put: (cmds at: i + 1).
		cmdMap at: (((cmds at: i) asciiValue - 96) + 1)	put: (cmds at: i + 1).
	].
	ShiftCmdActions _ cmdMap.! !


!Utilities class methodsFor: 'support windows' stamp: 'di 5/28/1998 12:11'!
commandKeyMappings
	^ self class firstCommentAt: #commandKeyMappings

"Lower-case command keys
a	Select all
b	Browse it
c	Copy
d	Do it
e	Exchange
f	Find
g	Find again
h	Set Search String
i	Inspect it
j	Again once
k	Set font
l	Cancel
m	Implementors of it
n	Senders of it
o	Spawn
p	Print it
q	Query symbol
r	Recognizer
s	Save (i.e. accept)
u	Align
v	Paste
w	Delete preceding word
x	Cut
y	Swap characters
z	Undo

Upper-case command keys (Hold down Cmd & Shift, or Ctrl key)
A	Advance argument
B	Browse it in this same browser (in System browsers only)
C	Compare argument to clipboard
D	Duplicate
E	Method strings containing it
F	Insert 'ifFalse:'
J	Again many
K	Set style
L	Outdent (move selection one tab-stop left)
N	References to it
R	Indent (move selection one tab-stap right)
S	Search
T	Insert 'ifTrue:'
U	Convert linefeeds to carriage returns in selection
V	Paste author's initials
W	Selectors containing it
X	Force selection to lowercase
Y	Force selection to uppercase
Z	Capitalize all words in selection
<return>		Insert return followed by as many tabs as the previous line
			(with a further adjustment for additional brackets in that line)

esc			Select current type-in
shift-delete	Forward delete character

Enclose the selection in a kind of bracket.  Each is a toggle.
Control-(	Enclose within ( and ), or remove enclosing ( and )
[	Enclose within [ and ], or remove enclosing [ and ]
{	Enclose within { and }, or remove enclosing { and }
<	Enclose within < and >, or remove enclosing < and >
'	Enclose within ' and ', or remove enclosing ' and '
""	Enclose within "" and "", or remove enclosing "" and ""
(Double click just inside any of the above delimiters to select the text inside it.)

Text Emphasis...
1	10 point font
2	12 point font
3	18 point font  (not in base image)
4	24 point font  (not in base image)
5	36 point font  (not in base image)

6	color, action-on-click, link to class comment, link to method, url
	Brings up a menu.  To remove these properties, select
	more than the active part and then use command-0.

7	bold
8	italic
9	narrow (same as negative kern)
0	plain text (resets all emphasis)
-	underlined (toggles it)
=	struck out (toggles it)

Cmd-shift
_ (aka shift -)	negative kern (letters 1 pixel closer)
+		positive kern (letters 1 pixel larger spread)
"
! !


"Postscript:
Install new command-key assignments"
ParagraphEditor initialize!

