'From Squeak 2.0 of May 22, 1998 on 15 June 1998 at 4:03:34 pm'!
"Change Set:		haloTweaks-sw
Date:			15 June 1998
Author:			Scott Wallace

Cleans up a variety of halo shortcomings, including:
  *  TextMorph text-editing handles relocated in order not to collide with rotate & help handle.
  *  Balloon help for halo handles is obtained from the handle's event handler rather than
          being inextricably and intractably correlated with the handle's color.
  *  Tweaks for positions and colors of halos.
  *  Tear-off-tile handle is readily touchable in all circumstances now."!


!Morph methodsFor: 'accessing' stamp: 'sw 6/15/1998 15:25'!
balloonHelpTextForHandle: aHandle
	|  itsSelector |
	itsSelector _ aHandle eventHandler firstMouseSelector.
	#(	(startRot:with:				'Rotate')
		(startGrow:with:				'Change size') 
		(doDup:with:				'Duplicate')
		(doGrab:with:				'Pick up')
		(doMenu:with:				'Menu')
		(mouseDownOnHelpHandle:	'Help')
		(tearOffTile					'Make a Tile')
		(paintBackground			'Paint background')
		(openViewerForArgument	'View me')
		(dismiss						'Remove')
		(editDrawing				'Repaint')
		(startScale:with:				'Change scale') 
		(chooseFont					'Change font')
		(chooseStyle					'Change style')
		(chooseEmphasis				'Change emphasis')
		(makeNewDrawingWithin	'Paint new object'))  "This last one not currently in use"


	do:
		[:pair | itsSelector == pair first ifTrue: [^ pair last]].
	^ 'unknown halo handle'! !

!Morph methodsFor: 'jun98 additions' stamp: 'sw 6/15/1998 15:14'!
addFontHandlesTo: aHalo box: box
	"Add handles for controlling font, style, and emphasis choices to the halo"

	aHalo addHandleAt: (box bottomLeft + (18@0)) color: Color lightGreen on: #mouseDown send: #chooseFont to: self.
	aHalo addHandleAt: (box bottomLeft + (36@0)) color: Color lightRed on: #mouseDown send: #chooseStyle to: self.
	aHalo addHandleAt: (box bottomRight - (18@0)) color: Color lightBrown darker on: #mouseDown send: #chooseEmphasis to: self.

! !


!HaloMorph methodsFor: 'private' stamp: 'sw 6/15/1998 14:54'!
addHandleAt: aPoint color: aColor on: eventName send: selector to: recipient
	"Add a handle centered at the given point with the given color, and set it up to respond to the given event by sending the given selector to the given recipient.  Return the handle."
	| handle |
	handle _ EllipseMorph
		newBounds: (Rectangle center: aPoint extent: 16@16)
		color: aColor.
	self addMorph: handle.
	handle on: #mouseUp send: #endInteraction to: self.
	handle on: eventName send: selector to: recipient.
	handle setBalloonText: (target balloonHelpTextForHandle: handle).
	^ handle
! !

!HaloMorph methodsFor: 'private' stamp: 'sw 6/15/1998 15:41'!
addHandles

	| box |
	target isWorldMorph ifTrue: [^ self addHandlesForWorldHalos].
	self removeAllMorphs.  "remove old handles, if any"
	self bounds: target fullBoundsInWorld.  "update my size"
	box _ (self fullBounds expandBy: 17)
			intersect: (self world bounds insetBy: 5@5).

	self addHandleAt: box topLeft color: Color red on: #mouseDown send: #doMenu:with: to: self.

	self addHandleAt: (box topLeft + (0 @ (9 + ((box height - 36) // 4)))) color: Color lightBrown on: #mouseDown send: #tearOffTile to: innerTarget.

	self addHandleAt: (box topLeft + (18@0)) color: Color transparent on: #mouseDown send: #dismiss to: self.
	self addHandleAt: (box leftCenter) color: Color cyan on: #mouseDown send: #openViewerForArgument to: innerTarget.

	self addHandleAt: box topCenter color: Color black on: #mouseDown send: #doGrab:with: to: self.

	self addHandleAt: box topRight color: Color green on: #mouseDown send: #doDup:with: to: self.

	target balloonText ifNotNil:
		[(self addHandleAt: box bottomCenter color: Color lightBlue on: #mouseDown send: #mouseDownOnHelpHandle: to: innerTarget)
			on: #mouseUp send: #deleteBalloon to: innerTarget].

	(self addHandleAt: box bottomLeft color: Color blue on: #mouseDown send: #startRot:with: to: self)
		on: #mouseStillDown send: #doRot:with: to: self.

	target isFlexMorph
		ifTrue:
			[(self addHandleAt: box bottomRight color: Color lightOrange on: #mouseDown send: #startScale:with: to: self)
				on: #mouseStillDown send: #doScale:with: to: self]
		ifFalse:
			[(self addHandleAt: box bottomRight color: Color yellow on: #mouseDown send: #startGrow:with: to: self)
				on: #mouseStillDown send: #doGrow:with: to: self].

	innerTarget addOptionalHandlesTo: self box: box.
	self addNameBeneath: box string: target "innerTarget" externalName.
	growingOrRotating _ false.
	self layoutChanged.
	self changed.
! !

!HaloMorph methodsFor: 'private' stamp: 'sw 6/15/1998 15:00'!
addHandlesForWorldHalos

	| box |
	self removeAllMorphs.  "remove old handles, if any"
	self bounds: target bounds.
	box _ self world bounds insetBy: 9.

	self addHandleAt: box topLeft color: Color red on: #mouseDown send: #doMenu:with: to: self.
	self addHandleAt: (box topLeft + (0@20)) color: Color lightBrown on: #mouseDown send: #tearOffTile to: target.
	self addHandleAt: box leftCenter color: Color cyan on: #mouseDown send: #openViewerForArgument to: target.

	target balloonText ifNotNil:
		[(self addHandleAt: box bottomCenter color: Color lightBlue on: #mouseDown send: #mouseDownOnHelpHandle: to: target)
			on: #mouseUp send: #deleteBalloon to: target].

	innerTarget addOptionalHandlesTo: self box: box.
	self addNameBeneath: (box insetBy: (0@0 corner: 0@10)) string: innerTarget externalName.
	growingOrRotating _ false.
	self layoutChanged.
	self changed.
! !


!PasteUpMorph methodsFor: 'menu & halo' stamp: 'sw 6/15/1998 15:01'!
addOptionalHandlesTo: aHalo box: box
	aHalo addHandleAt: box rightCenter color: Color lightGray on: #mouseDown send: #paintBackground to: self.
"
	aHalo addHandleAt: box leftCenter color: Color veryVeryLightGray on: #mouseDown send: #makeNewDrawingWithin to: self."! !


!SketchMorph methodsFor: 'other' stamp: 'sw 6/15/1998 15:01'!
addOptionalHandlesTo: aHalo box: box
	aHalo addHandleAt: box rightCenter color: Color lightGray on: #mouseDown send: #editDrawing to: aHalo innerTarget
! !


!StringMorph methodsFor: 'initialization' stamp: 'sw 6/15/1998 15:34'!
addOptionalHandlesTo: aHalo box: box
	self flag: #deferred.

	"Eventually...
	self addFontHandlesTo: aHalo box: box"! !


!TextMorph methodsFor: 'menu' stamp: 'sw 6/15/1998 15:13'!
addOptionalHandlesTo: aHalo box: box
	self addFontHandlesTo: aHalo box: box! !


HaloMorph removeSelector: #addHandleAt:color:!
