'From Squeak 2.4c of May 10, 1999 on 21 June 1999 at 4:55:56 pm'!
"Change Set:		flapAdvances-sw
Date:			21 June 1999
Author:			Scott Wallace

Numerous improvements and bug fixes relating to flaps, of which some are:

Flap tabs can now be dragged for positioning and flap-resizing whether or not the flap is open.

When switching among textual, graphical, and solid choices, former choices are honored.  (e.g. when you switch back to using a textual tab, the wording deployed when the tab last was textual is restored.)

When you dismiss a flap tab via its halo, the corresponding flap is also dismissed.

Flap-explanation window contents rewritten.

Flap menu changes:
*  Can now be persistent (very useful!!)
*  Menu items and title are self-updating.
*  Tab-color and flap-color can both be changed.
*  'Slide' and 'Inboard' options removed from the UI.
"!

ReferenceMorph subclass: #FlapTab
	instanceVariableNames: 'flapShowing edgeToAdhereTo slidesOtherObjects popOutOnDragOver popOutOnMouseOver inboard dragged lastReferentThickness '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Palettes'!

!Morph methodsFor: 'geometry' stamp: 'sw 6/18/1999 12:34'!
shiftSubmorphsBy: delta
	self shiftSubmorphsOtherThan: (submorphs select: [:m | (m hasProperty: #flap) or: [m isKindOf: FlapTab]]) by: delta! !


!PasteUpMorph methodsFor: 'flaps' stamp: 'sw 6/17/1999 16:02'!
assureFlapTabsFitOnScreen
	self flapTabs do:
		[:m | m fitOnScreen]! !


!ReferenceMorph methodsFor: 'menu' stamp: 'sw 6/17/1999 13:45'!
addCustomMenuItems: aCustomMenu hand: aHandMorph
	self isCurrentlyTextual
		ifTrue:
			[aCustomMenu add: 'change tab wording...' action: #changeTabText.
			aCustomMenu add: 'use graphical tab' action: #useGraphicalTab]
		ifFalse:
			[aCustomMenu add: 'use textual tab' action: #useTextualTab.
			aCustomMenu add: 'change tab graphic' action: #changeTabGraphic]! !

!ReferenceMorph methodsFor: 'menu' stamp: 'sw 6/21/1999 11:43'!
existingWording
	^ submorphs first contents asString! !

!ReferenceMorph methodsFor: 'menu' stamp: 'sw 6/17/1999 13:45'!
isCurrentlyTextual
	| first |
	^ ((first _ submorphs first) isKindOf: StringMorph) or: [first isKindOf: TextMorph]
		! !

!ReferenceMorph methodsFor: 'menu' stamp: 'sw 6/17/1999 13:46'!
preserveDetails
	"The receiver is being switched to use a different format.  Preserve the existing details (e.g. wording if textual, grapheme if graphical) so that if the user reverts back to the current format, the details will be right"

	self isCurrentlyTextual
		ifTrue:
			[self setProperty: #priorWording toValue: self existingWording]
		ifFalse:
			[self setProperty: #priorGraphic toValue: submorphs first form]! !

!ReferenceMorph methodsFor: 'menu' stamp: 'sw 6/21/1999 11:11'!
useGraphicalTab
	| aGraphic formToUse |
	self preserveDetails.
	self color: Color transparent.
	formToUse _ self valueOfProperty: #priorGraphic ifAbsent: [ScriptingSystem formAtKey: 'squeakyMouse'].
	aGraphic _ SketchMorph new form: formToUse.
	self borderWidth: 0.
	self removeAllMorphs.
	self addMorphBack: aGraphic.
	aGraphic position: self position.
	aGraphic lock.
	self fitContents.
	self layoutChanged.
	(owner isKindOf: IndexTabs) ifTrue:
		[owner laySubpartsOutInOneRow.
		isHighlighted ifTrue: [self highlight]].! !

!ReferenceMorph methodsFor: 'menu' stamp: 'sw 6/21/1999 11:51'!
useTextualTab
	| aLabel stringToUse |
	self preserveDetails.
	stringToUse _ self valueOfProperty: #priorWording ifAbsent: [self externalName].
	aLabel _ StringMorph  new contents: stringToUse.
	self replaceSubmorph: submorphs first by: aLabel.
	aLabel position: self position.
	aLabel highlightColor: self highlightColor; regularColor: self regularColor.
	aLabel lock.
	self fitContents.
	self layoutChanged.
	(owner isKindOf: IndexTabs) ifTrue:
		[self borderWidth: 0.
		owner laySubpartsOutInOneRow.
		isHighlighted ifTrue:
			[self highlight]]! !


!FlapTab commentStamp: 'sw 6/20/1999 23:01' prior: 0!
The tab associated with a flap.

nb: slidesOtherObjects and inboard are instance variables relating to disused features.  The feature implementations still exist in the system, but the UI to them has been sealed off.!

!FlapTab reorganize!
('initialization' adaptToWorld initialize preserveDetails)
('access' flapShowing isGlobal orientation permitsThumbnailing referentThickness tabThickness)
('parts bin' partsBinString togglePartsBinMode)
('edge' edgeString edgeToAdhereTo edgeToAdhereTo: setEdge: setEdgeToAdhereTo)
('menu' addAddHandMenuItemsForHalo:hand: addCustomMenuItems:hand: addTitleForHaloMenu: applyThickness: changeColor changeFlapColor destroyFlap dismissViaHalo flapMenuTitle thicknessString)
('mouseover & dragover' arrangeToPopOutOnDragOver: arrangeToPopOutOnMouseOver: dragoverString mouseoverString setToPopOutOnDragOver: setToPopOutOnMouseOver: toggleDragOverBehavior toggleMouseOverBehavior)
('positioning' adjustPositionVisAVisFlap fitOnScreen layoutChanged mouseMove: mouseUp: positionObject: positionReferent spanWorld stickOntoReferent transposeParts)
('show & hide' adjustPositionAfterHidingFlap hideFlap hideFlapUnlessBearingHalo hideFlapUnlessOverReferent maybeHideFlapOnMouseLeave maybeHideFlapOnMouseLeaveDragging showFlap tabSelected)
('textual tabs' assumeString:font:orientation:color: changeTabText existingWording isCurrentlyTextual textualTab textualTabString useStringTab: useTextualTab)
('graphical tabs' graphicalTab graphicalTabString isCurrentlyGraphical)
('solid tabs' applyTabThickness: changeTabSolidity changeTabThickness isCurrentlySolid notSolid solidTab solidTabString useSolidTab)
('disused options' inboard inboard: inboardString slideString slidesOtherObjects toggleInboardness toggleSlideBehavior)
!


!FlapTab methodsFor: 'initialization' stamp: 'sw 6/20/1999 19:17'!
preserveDetails
	"The receiver is being switched to use a different format.  Preserve the existing details (e.g. wording if textual, grapheme if graphical) so that if the user reverts back to the current format, the details will be right"

	| thickness |
	color = Color transparent ifFalse: [self setProperty: #priorColor toValue: color].
	self isCurrentlyTextual
		ifTrue:
			[self setProperty: #priorWording toValue: self existingWording]
		ifFalse:
			[self isCurrentlyGraphical
				ifTrue:
					[self setProperty: #priorGraphic toValue: submorphs first form]
				ifFalse:
					[thickness _ (self orientation == #vertical)
						ifTrue:	[self width]
						ifFalse:	[self height].
					self setProperty: #priorThickness toValue: thickness]]! !

!FlapTab methodsFor: 'access' stamp: 'sw 6/18/1999 13:38'!
referentThickness
	^ (self orientation == #horizontal)
		ifTrue:
			[referent height]
		ifFalse:
			[referent width]! !

!FlapTab methodsFor: 'parts bin' stamp: 'sw 6/17/1999 14:42'!
partsBinString

	^ referent isPartsBin
		ifTrue:	['suspend parts-bin behavior']
		ifFalse:	['bestow parts-bin behavior']! !

!FlapTab methodsFor: 'edge' stamp: 'sw 6/21/1999 11:13'!
edgeString
	^ 'cling to edge... (current: ', edgeToAdhereTo, ')'! !

!FlapTab methodsFor: 'edge' stamp: 'sw 6/18/1999 12:25'!
setEdge: anEdge
	| changedOrientation |
	changedOrientation _ nil.
	self orientation == #vertical
			ifTrue:
				[(#(top bottom) includes: anEdge) ifTrue:
					[changedOrientation _ #horizontal]]
			ifFalse:
				[(#(top bottom) includes: anEdge) ifFalse:
					[changedOrientation _ #vertical]].
	changedOrientation ifNotNil:
		[^ self inform:
'SORRY -- this sort of switch from horizontal to vertical
or vice-versa is not yet permitted.  For now, to achieve
the desired effect, create a new flap and copy over
the elements you want.'].

	self isCurrentlyTextual ifTrue:
		[changedOrientation ifNotNil:
			[self assumeString: (submorphs first contents copyWithout: Character cr) font: ScriptingSystem fontForScriptorButtons orientation: changedOrientation color: self color]].

	self edgeToAdhereTo: anEdge.
	changedOrientation ifNotNil:
		[self transposeParts].
	referent isInWorld ifTrue: [self positionReferent].
	self adjustPositionVisAVisFlap
! !

!FlapTab methodsFor: 'menu' stamp: 'sw 6/20/1999 22:30'!
addAddHandMenuItemsForHalo: aMenu hand: aHandMorph
	aMenu add: 'tab color...' target: self action: #changeColor.
	aMenu add: 'flap color...' target: self action: #changeFlapColor


! !

!FlapTab methodsFor: 'menu' stamp: 'sw 6/21/1999 11:36'!
addCustomMenuItems: aMenu hand: aHandMorph

	aMenu addLine.
	aMenu addUpdating: #textualTabString action: #textualTab.
	aMenu addUpdating: #graphicalTabString action: #graphicalTab.
	aMenu addUpdating: #solidTabString enablement: #notSolid action: #solidTab.
	aMenu addLine.

	(referent isKindOf: PasteUpMorph) ifTrue: 
		[aMenu addUpdating: #partsBinString action: #togglePartsBinMode].
	aMenu addUpdating: #dragoverString action: #toggleDragOverBehavior.
	aMenu addUpdating: #mouseoverString action: #toggleMouseOverBehavior.
	"aMenu addUpdating: #slideString action: #toggleSlideBehavior.
	aMenu addUpdating: #inboardString action: #toggleInboardness."
	"aMenu addUpdating: #thicknessString ('thickness... (current: ', self thickness printString, ')') action: #setThickness."
	aMenu addUpdating: #edgeString action: #setEdgeToAdhereTo.
	aMenu addLine.
	aMenu add: 'destroy this flap' action: #destroyFlap.! !

!FlapTab methodsFor: 'menu' stamp: 'sw 6/19/1999 23:16'!
addTitleForHaloMenu: aMenu
	aMenu addTitle: self externalName updatingSelector: #flapMenuTitle updateTarget: self! !

!FlapTab methodsFor: 'menu' stamp: 'sw 6/20/1999 23:41'!
applyThickness: newThickness
	| toUse |
	toUse _ newThickness asNumber max: 0.
	(self orientation == #vertical)
			ifTrue:
				[referent width: toUse]
			ifFalse:
				[referent height: toUse].
	self positionReferent. 
	self adjustPositionVisAVisFlap! !

!FlapTab methodsFor: 'menu' stamp: 'sw 6/20/1999 22:29'!
changeColor
	self isCurrentlyGraphical
		ifTrue:
			[^ self inform: 'Color only pertains to a flap tab when the 
tab is textual or "solid".  This tab is
currently graphical, so color-choice
does not apply.' ].
	super changeColor
	
! !

!FlapTab methodsFor: 'menu' stamp: 'sw 6/20/1999 23:14'!
changeFlapColor
	(self flapShowing)
		ifTrue:
			[referent changeColor]
		ifFalse:
			[self inform: 'The flap itself needs to be open
before you can change its
color.']! !

!FlapTab methodsFor: 'menu' stamp: 'sw 6/17/1999 14:20'!
dismissViaHalo
	referent delete.
	self delete.! !

!FlapTab methodsFor: 'menu' stamp: 'sw 6/19/1999 23:23'!
flapMenuTitle
	^ 'flap: ', (self isCurrentlyTextual
		ifTrue:
			[self existingWording]
		ifFalse:
			[self valueOfProperty: #priorWording ifAbsent: '---'])! !

!FlapTab methodsFor: 'menu' stamp: 'sw 6/14/1999 16:38'!
thicknessString
	^ 'thickness... (currently ', self thickness printString, ')'! !

!FlapTab methodsFor: 'mouseover & dragover' stamp: 'sw 6/14/1999 16:32'!
dragoverString

	^ popOutOnDragOver
		ifTrue:
			['cease popping out on dragover']
		ifFalse:
			['start popping out on dragover']! !

!FlapTab methodsFor: 'mouseover & dragover' stamp: 'sw 6/21/1999 11:14'!
mouseoverString
	^ popOutOnMouseOver
		ifTrue:
			['cease popping out on mouseover']
		ifFalse:
			['start popping out on mouseover']! !

!FlapTab methodsFor: 'positioning' stamp: 'sw 6/17/1999 16:25'!
fitOnScreen
	self orientation == #vertical
			ifTrue:
				[self top: (self top min: (self bottom- self height))]
			ifFalse:
				[self left: (self left min: (self right - self width))]! !

!FlapTab methodsFor: 'positioning' stamp: 'sw 6/17/1999 16:07'!
layoutChanged
	self fitOnScreen.
	self adaptToWorld.
	super layoutChanged! !

!FlapTab methodsFor: 'positioning' stamp: 'sw 6/20/1999 21:31'!
mouseMove: evt
	| aPosition newReferentThickness adjustedPosition thick |

	dragged ifFalse: [(thick _ self referentThickness) > 0
			ifTrue: [lastReferentThickness _ thick]].
	((self containsPoint: (aPosition _ evt cursorPoint)) and: [dragged not])
		ifFalse:
			[flapShowing ifFalse: [self showFlap].
			adjustedPosition _ aPosition - evt hand targetOffset.
			(edgeToAdhereTo == #bottom)
				ifTrue:
					[newReferentThickness _ inboard
						ifTrue:
							[self world height - adjustedPosition y]
						ifFalse:
							[self world height - adjustedPosition y - self height]].

			(edgeToAdhereTo == #left)
					ifTrue:
						[newReferentThickness _
							inboard
								ifTrue:
									[adjustedPosition x + self width]
								ifFalse:
									[adjustedPosition x]].

			(edgeToAdhereTo == #right)
					ifTrue:
						[newReferentThickness _
							inboard
								ifTrue:
									[self world width - adjustedPosition x]
								ifFalse:
									[self world width - adjustedPosition x - self width]].

			(edgeToAdhereTo == #top)
					ifTrue:
						[newReferentThickness _
							inboard
								ifTrue:
									[adjustedPosition y + self height]
								ifFalse:
									[adjustedPosition y]].
		
			self isCurrentlySolid ifFalse:
				[(#(left right) includes: edgeToAdhereTo)
					ifFalse:
						[self left: adjustedPosition x]
					ifTrue:
						[self top: adjustedPosition y]].

			self applyThickness: newReferentThickness.

			dragged _ true]! !

!FlapTab methodsFor: 'positioning' stamp: 'sw 6/18/1999 13:51'!
mouseUp: evt
	super mouseUp: evt.
	self referentThickness <= 0 ifTrue: [flapShowing _ false].
	dragged _ false! !

!FlapTab methodsFor: 'show & hide' stamp: 'sw 6/21/1999 13:07'!
hideFlapUnlessOverReferent
	| aWorld |
	(referent isInWorld and: [referent bounds containsPoint: self world activeHand lastEvent cursorPoint])
		ifFalse:
			[aWorld _ self world.
			self referent delete.
			aWorld removeAccommodationForFlap: self.
			flapShowing _ false.
			self isInWorld ifFalse:
				[self inboard ifTrue: [aWorld addMorphFront: self]].
			self adjustPositionAfterHidingFlap]! !

!FlapTab methodsFor: 'show & hide' stamp: 'sw 6/18/1999 13:47'!
showFlap
	| aWorld thicknessToUse |

	aWorld _ self world.
	self referentThickness <= 0
		ifTrue:
			[thicknessToUse _ lastReferentThickness ifNil: [100].
			self orientation == #horizontal
				ifTrue:
					[referent height: thicknessToUse]
				ifFalse:
					[referent width: thicknessToUse]].
	inboard ifTrue:
		[self stickOntoReferent].  "makes referent my owner, and positions me accordingly"
	referent world == aWorld
		ifFalse:
			[aWorld accommodateFlap: self.  "Make room if needed"
			aWorld addMorphFront: referent.
			aWorld startSteppingSubmorphsOf: referent.
			self positionReferent.
			referent adaptToWorld: aWorld].
	inboard  ifFalse:
		[self adjustPositionVisAVisFlap].
	flapShowing _ true.
	
	aWorld bringFlapTabsToFront! !

!FlapTab methodsFor: 'textual tabs' stamp: 'sw 6/21/1999 11:09'!
assumeString: aString font: aFont orientation: anOrientation color: aColor
	| aTextMorph workString pad |
	pad _ (anOrientation == #vertical) 
		ifTrue:	[Character cr]
		ifFalse:	[$ ].
	workString _ pad asString.
	aString do:
		[:ch |
			workString _ workString copyWith: ch.
			workString _ workString copyWith: pad].
	(anOrientation == #vertical)
		ifTrue:
			[workString _ workString copyFrom: 2 to: workString size - 1].

	aTextMorph _ (TextMorph new beAllFont: aFont) width: 10; contents: workString; yourself.
	self removeAllMorphs.
	self addMorph: aTextMorph centered.
	aTextMorph lock.
	anOrientation == #horizontal
		ifTrue:
			[self borderWidth: 0]
		ifFalse:
			[self borderWidth: 3; borderColor: #raised].
	self fitContents.
	aColor ifNotNil: [self color: aColor].
	aTextMorph position: self position.
	self layoutChanged! !

!FlapTab methodsFor: 'textual tabs' stamp: 'sw 6/17/1999 13:34'!
changeTabText
	| reply |

	reply _ FillInTheBlankMorph request: 'new wording for this tab:' initialAnswer: self existingWording centerAt: self cursorPoint inWorld: self world.
	reply size > 0 ifFalse: [^ self].
	self useStringTab: reply.
	submorphs first delete.
	self assumeString: reply font: ScriptingSystem fontForScriptorButtons orientation: (Utilities orientationForEdge: edgeToAdhereTo) color: nil! !

!FlapTab methodsFor: 'textual tabs' stamp: 'sw 6/17/1999 13:33'!
existingWording
	| longForm |
	longForm _ submorphs first contents.
	^ self orientation == #vertical
		ifTrue:
			[longForm asString copyWithout: Character cr]
		ifFalse:
			[(longForm asString collectWithIndex:
				[:ch :i | i even ifFalse: [$] ifTrue: [ch]]) copyWithout: $]! !

!FlapTab methodsFor: 'textual tabs' stamp: 'sw 6/20/1999 19:19'!
isCurrentlyTextual
	| first |
	^ submorphs size > 0 and: [((first _ submorphs first) isKindOf: StringMorph) or: [first isKindOf: TextMorph]]! !

!FlapTab methodsFor: 'textual tabs' stamp: 'sw 6/17/1999 13:21'!
textualTab
	self isCurrentlyTextual
		ifTrue:
			[self changeTabText]
		ifFalse:
			[self useTextualTab]! !

!FlapTab methodsFor: 'textual tabs' stamp: 'sw 6/20/1999 19:09'!
textualTabString
	^ self isCurrentlyTextual
		ifTrue:
			['change tab wording...']
		ifFalse:
			['use textual tab']! !

!FlapTab methodsFor: 'textual tabs' stamp: 'sw 6/17/1999 13:51'!
useTextualTab
	| stringToUse |
	self preserveDetails.
	submorphs size > 0 ifTrue: [self removeAllMorphs].
	stringToUse _ self valueOfProperty: #priorWording ifAbsent: ['Unnamed Flap'].
	self assumeString: stringToUse font: ScriptingSystem fontForScriptorButtons orientation: self orientation color: Color green muchLighter.
! !

!FlapTab methodsFor: 'graphical tabs' stamp: 'sw 6/17/1999 16:07'!
graphicalTab
	self isCurrentlyGraphical
		ifTrue:
			[self changeTabGraphic]
		ifFalse:
			[self useGraphicalTab]! !

!FlapTab methodsFor: 'graphical tabs' stamp: 'sw 6/20/1999 19:10'!
graphicalTabString
	^ self isCurrentlyGraphical
		ifTrue:
			['choose new graphic...']
		ifFalse:
			['use graphical tab']! !

!FlapTab methodsFor: 'graphical tabs' stamp: 'sw 6/20/1999 19:20'!
isCurrentlyGraphical
	| first |
	^ submorphs size > 0 and:
		[((first _ submorphs first) isKindOf: ImageMorph) or: [first isKindOf: SketchMorph]]! !

!FlapTab methodsFor: 'solid tabs' stamp: 'sw 6/21/1999 11:40'!
changeTabSolidity
	"Presently no actual options associated with this menu item if the flap is currently alreadly solid, so entertain the user with an anuran sound.  However, in latest scheme, the corresponding menu item is disabled in this circumstance, so this method is effectively unreachable."

	self playSoundNamed: 'croak'! !

!FlapTab methodsFor: 'solid tabs' stamp: 'sw 6/21/1999 11:39'!
isCurrentlySolid
	"Don't never use double negatives"

	^ self notSolid not! !

!FlapTab methodsFor: 'solid tabs' stamp: 'sw 6/21/1999 11:36'!
notSolid
	"Answer whether the receiver is currenty not solid.  Used for determining whether the #solidTab menu item should be enabled"

	^ self isCurrentlyTextual or: [self isCurrentlyGraphical]! !

!FlapTab methodsFor: 'solid tabs' stamp: 'sw 6/20/1999 21:34'!
solidTab
	self isCurrentlySolid
		ifFalse:
			[self useSolidTab]
		ifTrue:
			[self changeTabSolidity]! !

!FlapTab methodsFor: 'solid tabs' stamp: 'sw 6/20/1999 19:09'!
solidTabString
	^ self isCurrentlySolid
		ifTrue:
			['currently using solid tab']
		ifFalse:
			['use solid tab']! !

!FlapTab methodsFor: 'solid tabs' stamp: 'sw 6/20/1999 20:55'!
useSolidTab
	| thickness colorToUse |
	self preserveDetails.

	thickness _ self valueOfProperty: #priorThickness ifAbsent: [20].
	colorToUse _ self valueOfProperty: #priorColor ifAbsent: [Color red muchLighter].
	self color: colorToUse.
	self removeAllMorphs.
	
	(self orientation == #vertical)
		ifTrue:
			[self width: thickness.
			self height: self currentWorld height.
			self position: (self position x @ 0)]
		ifFalse:
			[self height: thickness.
			self width: self currentWorld width.
			self position: (0 @ self position y)].

	self borderWidth: 0.
	self layoutChanged.! !

!FlapTab methodsFor: 'disused options' stamp: 'sw 6/21/1999 13:03'!
inboard
	^ inboard == true! !

!FlapTab methodsFor: 'disused options' stamp: 'sw 6/14/1999 16:37'!
inboardString
	^ inboard
		ifTrue:
			['switch to being outboard']
		ifFalse:
			['switch to being inboard']! !

!FlapTab methodsFor: 'disused options' stamp: 'sw 6/14/1999 16:35'!
slideString

	^ slidesOtherObjects
		ifTrue:
			['cease slide behavior']
		ifFalse:
			['start slide behavior']! !


!Utilities class methodsFor: 'flaps' stamp: 'sw 6/20/1999 22:56'!
explainFlaps
	"Open a window giving flap  help."
	"Utilities explainFlaps"
	| aString |
	aString _ 
'Flaps are like drawers on the edge of the screen, which can be opened so that you can use what is inside them, and closed when you do not need them.  They have many possible uses, a few of which are illustrated by the default set of flaps you can get as described below.

"Global flaps" are available in every morphic project.  As you move from project to project, you will see the same flaps in each.   To get started using global flaps, set the "useGlobalFlaps" Preference to true, or choose "start using global flaps" from the windows/flaps menu.

"Project flaps" are flaps that belong to a single morphic project.

If a flap is set up as a parts bin (such as the default Tools and Supplies flaps), you can use it to create new objects -- just mouse-over the tab so that the flap will open up, then find the object you want, and drag it; when the cursor leaves the flap, the flap itself will disappear, and you''ll be left holding the new object -- just click to place it exactly where you want it.

If a flap is *not* set up as a parts bin (such as the default "Squeak" flap at the left edge of the screen) you can park objects there (this is an easy way to move objects from project to project) and you can place your own private controls there, etc.  Everything in the default "Squeak" flap (and all the other default flaps, for that matter) is there only for illustrative purposes -- every user will want to fine-tune the flaps to suit his/her own style and needs.

Each flap may be set up to appear on mouseover, dragover, both, or neither.  See the menu items described below for more about these and other options.

You can open a closed flap by clicking on its tab, or by dragging the tab toward the center of the screen

You can close an open flap by clicking on its tab or by dragging the tab back off the edge of the screen.

Drag the tab of a flap to reposition the tab and to resize the flap itself.  Repositioning starts when you drag the cursor out of the original tab area.

If flaps or their tabs seem wrongly positioned or lost, try issuing a restoreDisplay from the screen menu.

The red-halo menu on a flap gives you access to the "flap..." submenu which allows you to change its properties.   For greatest ease of use, request "keep this menu up" here -- that way, you can easily explore all the options in the menu.

tab color...				Lets you change the color of the flap''s tab.
flap color...				Lets you change the color of the flap itself.

use textual tab...		If the tab is not textual, makes it become textual.
change tab wording...	If the tab is already textual, allows you to edit
							its wording.

use graphical tab...		If the tab is not graphical, makes it become
							graphical.
choose tab graphic...	If the tab is already graphical, allows you
							to change the picture.

use solid tab...			If the tab is not solid, makes it become solid, i.e.
							appear as a solid band of color along the
							entire length or width of the screen.

parts-bin behavior		If set, then dragging an object from the flap
							tears off a new copy of the object.

dragover				If set, the flap opens on dragover and closes
							again on drag-leave.

mouseover				If set, the flap opens on mouseover and closes
							again on mouse-leave. 

cling to edge...			Governs which edge (left, right, top, bottom)
							the flap adheres to.

destroy this flap		Deletes the flap.

To define a new flap, use "new global flap..." or "new project flap...", found in the "windows and flaps" menu.

To reinstate the default system flaps, evaluate "Utilities reinstateDefaultFlaps"  (caveat -- this will first remove all existing flaps, including any that you may have manually added or edited.)

If flaps that you wish to use appear to be buried behind other objects on your screen, simply clicking on the desktop background will bring all flap tabs to the front.

To add, delete, or edit things on a given flap, it is often wise first to suspend the flap''s mouse-over and drag-over sensitivity, so it won''t keep disappearing on you while you''re trying to work with it.

The "Menu" flap provided in the default flap-set provides a service that will be valuable to some (it gives a sense of place to many standard menu item, and allows you to avoid navigating through submenus) and dismaying to others (who should destroy the flap immediately -- do this by bringing up the red-halo menu for the "Menu" tab, and then choosing "destroy this flap").

'.
	
  

	(StringHolder new contents: aString)
		openLabel: 'Flaps in Morphic'

	! !


FlapTab removeSelector: #ensureSolidBounds!
FlapTab removeSelector: #graphicalTabStrilng!
FlapTab removeSelector: #offerFlapMenu!
"Postscript:
"
Utilities replaceMenuFlap.!

