'From Squeak 2.1 of June 30, 1998 on 22 August 1998 at 10:47:04 am'!

!ClassDescription methodsFor: 'initialize-release' stamp: 'tk 8/21/1998 10:01'!
validateFrom: oldClass in: environ instanceVariableNames: invalidFields methods: invalidMethods wasPresent: wasPresent
	"Recompile the receiver, a class, and redefine its subclasses if necessary.
	The parameter invalidFields is no longer really used"

	| newSub invalidSubMethods |
	oldClass becomeUncompact.  "Its about to be abandoned"
	invalidMethods & self hasMethods
		ifTrue: 
			[Transcript show: 'recompiling ' , self name , '...'.
			self compileAllFrom: oldClass.
			Transcript show: ' done'; cr].
	invalidSubMethods _ invalidMethods | (self instSize ~= oldClass instSize).
	self == oldClass
		ifTrue: [invalidSubMethods ifFalse: [^self]]
		ifFalse: [wasPresent ifTrue: [self updateInstancesFrom: oldClass]].
	oldClass subclasses do: 
		[:sub | 
		newSub _ sub copyForValidation.
		newSub
			subclassOf: self
			oldClass: sub
			instanceVariableNames: sub instVarNames
			variable: sub isVariable
			words: sub isBytes not
			pointers: sub isBits not
			ifBad: [self error: 'terrible problem in recompiling subclasses!!'].
		newSub
			validateFrom: sub
			in: environ
			instanceVariableNames: invalidFields
			methods: invalidSubMethods
			wasPresent: true]	"if false, no subclasses, does not get here"! !


!Class methodsFor: 'initialize-release' stamp: 'tk 8/21/1998 09:53'!
validateFrom: oldClass in: environ instanceVariableNames: invalidFields methods: invalidMethods wasPresent: wasPresent
	"Recompile the receiver and redefine its subclasses if necessary."

	super
		validateFrom: oldClass
		in: environ
		instanceVariableNames: invalidFields
		methods: invalidMethods
		wasPresent: wasPresent.
	self ~~ oldClass
		ifTrue: 
			[environ at: name put: self.
			oldClass obsolete]! !


!Metaclass commentStamp: 'tk 8/22/1998 10:47' prior: 0!
My instances add instance-specific behavior to various class-describing objects in the system. This typically includes messages for initializing class variables and instance creation messages particular to a class. There is only one instance of a particular Metaclass, namely the class which is being described. A Metaclass shares the class variables of its instance.
	
[Subtle] In general, the superclass hierarchy for metaclasses parallels that for classes. Thus,
	Integer superclass == Number, and
	Integer class superclass == Number class.
However there is a singularity at Object. Here the class hierarchy terminates, but the metaclass hierarchy must wrap around to Class, since ALL metaclasses are subclasses of Class. Thus,
	Object superclass == nil, and
	Object class superclass == Class.

[Subtle detail] A class is know by name to an environment.  Typically this is the SystemDictionary named Smalltalk.  If we ever make lightweight classes that are not in Smalltalk, they must be in some environment.  Specifically, the code that sets 'wasPresent' in name:inEnvironment:subclassOf:instanceVariableNames:variable:words:pointers:classVariableNames:poolDictionaries:category:comment:changed: must continue to work.!

!Metaclass methodsFor: 'initialize-release' stamp: 'tk 8/21/1998 10:03'!
instanceVariableNames: instVarString 
	"Declare additional named variables for my instance."
	| newMeta invalid |
	newMeta _ self copyForValidation.
	invalid _ newMeta
				subclassOf: superclass
				oldClass: self
				instanceVariableNames: instVarString
				variable: false
				words: true
				pointers: true
				ifBad: [^false].
	(invalid or: [instVarString ~= self instanceVariablesString])
		ifTrue: [newMeta validateFrom: self
					in: Smalltalk
					instanceVariableNames: true
					methods: true
					wasPresent: true.	"as far as we know"
				Smalltalk changes changeClass: self]! !

!Metaclass methodsFor: 'initialize-release' stamp: 'tk 8/22/1998 07:45'!
updateInstancesFrom: oldClass 
	"Recreate any existing instances of the argument, oldClass, as instances of the receiver, which is a newly changed class. oldClass is a metaclass, so we know there is just one instance.  Permute variables as necessary."

	| oldInstVarNames map variable new instSize oldInstances |
	oldClass soleInstance == nil ifTrue: [^self].	"no instances to convert"
	oldInstVarNames _ oldClass allInstVarNames.
	map _ self allInstVarNames 
			collect: [:instVarName | oldInstVarNames indexOf: instVarName].
	variable _ self isVariable.
	instSize _ self instSize.

	"Now perform a bulk mutation of old instances into new ones"
	oldInstances _ Array with: oldClass soleInstance.
	oldInstances elementsExchangeIdentityWith:
		(oldInstances collect: 
		[:old | 
		variable
			ifTrue: [new _ self basicNew: old basicSize]
			ifFalse: [new _ self basicNew].
		1 to: instSize do: 
			[:offset |  (map at: offset) > 0 ifTrue:
				[new instVarAt: offset
						put: (old instVarAt: (map at: offset))]].
		variable 
			ifTrue: [1 to: old basicSize do: 
						[:offset |
						new basicAt: offset put: (old basicAt: offset)]].
		new])! !

!Metaclass methodsFor: 'class hierarchy' stamp: 'tk 8/22/1998 10:46'!
name: newName inEnvironment: environ subclassOf: sup instanceVariableNames: instVarString variable: v words: w pointers: p classVariableNames: classVarString poolDictionaries: poolString category: categoryName comment: commentString changed: changed 
	"This is the standard initialization message for creating a new Metaclass. 
	Answer an instance of me from the information provided in the 
	arguments. Create an error notification if the name does not begin with 
	an uppercase letter or if a class of the same name already exists.
	1/22/96 sw: don't ever do addClass, always do changeClass"

	| wasPresent oldClass newClass invalidFields invalidMethods |
	newName first isUppercase
		ifFalse: 
			[self error: 'Class names must be capitalized'.
			^false].
	(wasPresent _ environ includesKey: newName)
		ifTrue: 
			[oldClass _ environ at: newName.
			(oldClass isKindOf: Behavior)
				ifFalse: 
					[self error: newName , ' already exists!!  Proceed will store over it'.
					wasPresent _ false.
					oldClass _ self newNamed: newName]]
		ifFalse: [oldClass _ self newNamed: newName.
				Smalltalk flushClassNameCache].
	newClass _ oldClass copy.
	invalidFields _ changed | (newClass
					subclassOf: sup
					oldClass: oldClass
					instanceVariableNames: instVarString
					variable: v
					words: w
					pointers: p
					ifBad: [^false]).
	invalidFields not & (oldClass instSize = newClass instSize)
		ifTrue: [newClass _ oldClass].
	invalidMethods _ invalidFields | (newClass declare: classVarString) | 
		(newClass sharing: poolString).
	commentString == nil ifFalse: [newClass comment: commentString].
	(environ includesKey: newName)
		ifFalse: [environ declare: newName from: Undeclared].
	environ at: newName put: newClass.
	SystemOrganization classify: newClass name under: categoryName asSymbol.
	newClass
		validateFrom: oldClass
		in: environ
		instanceVariableNames: invalidFields
		methods: invalidMethods
		wasPresent: wasPresent.
	"update subclass lists"
	newClass superclass removeSubclass: oldClass.
	newClass superclass addSubclass: newClass.
	"Update Changes"
	Smalltalk changes changeClass: newClass.
	^ newClass! !


!Object class methodsFor: 'instance creation' stamp: 'tk 8/22/1998 08:22'!
initialInstance
	"Answer the first instance of the receiver, generate an error if there is one already"
	"self instanceCount > 0 ifTrue: [self error: 'instance(s) already exist.']."
		"Debugging test that is very slow"
	^ self new! !

!Object class methodsFor: 'instance creation' stamp: 'tk 8/22/1998 08:27'!
instanceOfUniqueClassWithInstVarString: instVarString andClassInstVarString: classInstVarString
	"Create a unique class for the receiver, and answer an instance of it"

	^ (self newUniqueClassInstVars: instVarString 
		classInstVars: classInstVarString) initialInstance! !


ClassDescription removeSelector: #validateFrom:in:instanceVariableNames:methods:!
Class removeSelector: #validateFrom:in:instanceVariableNames:methods:!
