'From Squeak3.0 of 4 February 2001 [latest update: #3552] on 21 February 2001 at 11:42:26 pm'!
"Change Set:		ScrollingEnh
Date:			21 February 2001
Author:			Jesse Welton

Enhancements to scrolling that tidy up the various xxxScrollRange methods, and to make ScrollPanes more responsive to changes in the scroll range.  Also eliminates the occasional one-pixel offset errors that can result from (implicitly) truncating the offset rather than rounding it."!


!ScrollPane methodsFor: 'initialization' stamp: 'JW 2/21/2001 23:03'!
setScrollDeltas
	"Set the ScrollBar deltas, value and interval, based on the current scroll pane size, offset and range."
	| range delta |
	self hideOrShowScrollBar.
	scroller hasSubmorphs ifFalse: [scrollBar interval: 1.0.  ^ self].
	scroller fullBounds. "force recompute so that leftoverScrollRange will be up-to-date"
	delta _ self scrollDeltaHeight.
	range _ self leftoverScrollRange.
	range = 0 ifTrue: [^ scrollBar scrollDelta: 0.02 pageDelta: 0.2; interval: 1.0; setValue: 0].

	"Set up for one line (for arrow scrolling), or a full pane less one line (for paging)."
	scrollBar scrollDelta: (delta / range) asFloat 
			pageDelta: ((self innerBounds height - delta) / range) asFloat.
	scrollBar interval: ((self innerBounds height) / self totalScrollRange) asFloat.
	scrollBar setValue: (scroller offset y / self leftoverScrollRange min: 1.0) asFloat.! !

!ScrollPane methodsFor: 'geometry' stamp: 'JW 2/21/2001 22:06'!
extraScrollRange
	"Return the amount of extra blank space to include below the bottom of the scroll content."
	"The classic behavior would be ^bounds height - (bounds height * 3 // 4)"
	^ self scrollDeltaHeight! !

!ScrollPane methodsFor: 'geometry' stamp: 'JW 2/21/2001 22:51'!
leftoverScrollRange
	"Return the entire scrolling range minus the currently viewed area."
	scroller submorphBounds ifNil: [^ 0].
	^ (self totalScrollRange - bounds height roundTo: self scrollDeltaHeight) max: 0
! !

!ScrollPane methodsFor: 'geometry' stamp: 'JW 2/21/2001 22:50'!
totalScrollRange
	"Return the entire scrolling range."
	^ self unadjustedScrollRange + self extraScrollRange! !

!ScrollPane methodsFor: 'geometry' stamp: 'JW 2/21/2001 22:49'!
unadjustedScrollRange
	"Return the height extent of the receiver's submorphs."
	| submorphBounds |
	submorphBounds := scroller submorphBounds ifNil: [^ 0].
	^ submorphBounds bottom
! !

!ScrollPane methodsFor: 'scroll bar events' stamp: 'JW 2/21/2001 22:44'!
scrollBarValue: scrollValue
	scroller hasSubmorphs ifFalse: [^ self].
	scroller offset: -3 @ (self leftoverScrollRange * scrollValue) rounded! !

!ScrollPane methodsFor: 'scrolling' stamp: 'JW 2/21/2001 22:43'!
isScrollable
	(Preferences valueOfFlag: #hiddenScrollBars) ifFalse: [^ true].

	"If the contents of the pane are too small to scroll, return false."
	^ self leftoverScrollRange > 0
		"treat a single line as non-scrollable"
		and: [self totalScrollRange > (self scrollDeltaHeight * 3/2)]! !


!AlansTextPlusMorph methodsFor: 'as yet unclassified' stamp: 'JW 2/21/2001 22:54'!
extraScrollRange
	^ bounds height
! !


!PluggableTextMorph methodsFor: 'geometry' stamp: 'JW 2/21/2001 22:15'!
extraScrollRange
	^ self height // 4! !

AlansTextPlusMorph removeSelector: #leftoverScrollRange!
