'From Squeak 2.3 of January 14, 1999 on 11 April 1999 at 1:15:06 pm'!
"Change Set:		ShrinkAssist
Date:			11 April 1999
Author:			Dan Ingalls

Updates the Squeak shrink routines to assist with reducing the size of Squeak images.
"!


!Integer methodsFor: 'benchmarks' stamp: 'di 4/11/1999 11:20'!
benchmark  "Handy bytecode-heavy benchmark"
	"(500000 // time to run) = approx bytecodes per second"
	"5000000 // (Time millisecondsToRun: [10 benchmark]) * 1000"
	"3059000 on a Mac 8100/100"
    | size flags prime k count |
    size _ 8190.
    1 to: self do:
        [:iter |
        count _ 0.
        flags _ (Array new: size) atAllPut: true.
        1 to: size do:
            [:i | (flags at: i) ifTrue:
                [prime _ i+1.
                k _ i + prime.
                [k <= size] whileTrue:
                    [flags at: k put: false.
                    k _ k + prime].
                count _ count + 1]]].
    ^ count! !


!SystemDictionary methodsFor: 'shrinking' stamp: 'di 4/11/1999 11:45'!
discard3D
	"Discard 3D Support."

	Smalltalk removeKey: #WonderlandConstants ifAbsent: [].
	Smalltalk removeKey: #B3DEngineConstants ifAbsent: [].
	SystemOrganization removeCategoriesMatching: 'Morphic-Balloon-Demos'.
	SystemOrganization removeCategoriesMatching: 'Balloon-3D-*'.
	SystemOrganization removeCategoriesMatching: 'Wonderland-*'.

! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'di 4/11/1999 12:36'!
discardFlash
	"Discard Flash and TrueType support."

	SystemOrganization removeCategoriesMatching: 'MM-Flash-*'.
	SystemOrganization removeCategoriesMatching: 'TrueType-*'.

! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'di 4/11/1999 12:09'!
discardMorphic
	"Discard Morphic."
	| subs |
	Smalltalk discard3D.
	Smalltalk discardFlash.
	subs _ OrderedCollection new.
	Morph allSubclassesWithLevelDo: [:c :i | subs addFirst: c]
		startingLevel: 0.
	subs do: [:c | c removeFromSystem].
	Smalltalk removeKey: #BalloonEngineConstants ifAbsent: [].
	SystemOrganization removeCategoriesMatching: 'User Objects'.
	SystemOrganization removeCategoriesMatching: 'Experimental-*'.
	SystemOrganization removeCategoriesMatching: 'Balloon-*'.
	SystemOrganization removeCategoriesMatching: 'Morphic-*'.

! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'di 4/11/1999 11:39'!
discardNetworking
	"Discard the support for TCP/IP networking."

	SystemOrganization removeCategoriesMatching: 'Interface-Web Browser'.
	SystemOrganization removeCategoriesMatching: 'Interface-Mail Reader'.
	SystemOrganization removeCategoriesMatching: 'Interface-IRC Chat'.
	Smalltalk discardPluggableWebServer.
	SystemOrganization removeCategoriesMatching: 'HTML-*'.
	SystemOrganization removeCategoriesMatching: 'NetTools-*'.
	SystemOrganization removeCategoriesMatching: 'Network-*'.
	SystemOrganization removeCategoriesMatching: 'System-Network'.
! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'di 4/11/1999 12:40'!
discardSoundSynthesis
	"Discard the sound synthesis facilities, and the methods and classes that use it. This also discards MIDI."

	Smalltalk discardMIDI.
	Smalltalk removeClassNamed: #EnvelopeLineMorph.
	Smalltalk removeClassNamed: #EnvelopeEditorMorph.
	Smalltalk removeClassNamed: #PianoKeyboardMorph.
	Smalltalk removeClassNamed: #WaveEditor.
	Smalltalk removeClassNamed: #SoundSequencerMorph.
	Smalltalk removeClassNamed: #SoundMorph.
	Smalltalk removeClassNamed: #SoundLoopMorph.
	Smalltalk removeClassNamed: #InterimSoundMorph.
	Smalltalk removeClassNamed: #RecordingControlsMorph.
	Smalltalk removeClassNamed: #PermanentRecordingControlsMorph.
	Smalltalk removeClassNamed: #SoundDemoMorph.
	Smalltalk at: #GraphMorph ifPresent: [:graphMorph |
		#(loadCoffeeCupClink play playBach playOnce
		  readDataFromFile registerWaveform stopPlaying)
			do: [:sel | graphMorph removeSelector: sel]].
	Smalltalk at: #TrashCanMorph ifPresent: [:trashMorph |
		trashMorph class removeSelector: #samplesForDelete.
		trashMorph class removeSelector: #samplesForMouseEnter.
		trashMorph class removeSelector: #samplesForMouseLeave].
	SystemOrganization removeCategoriesMatching: 'Music-Synthesis'.
! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'di 4/11/1999 13:13'!
majorShrink    "Smalltalk majorShrink; abandonSources; lastRemoval"
	"This method throws out lots of the system that is not needed for, eg, operation in a hand-held PC.  Currently, since it discards Morphic, majorShrink must be run in an MVC project.  Moreover, since it throws out projects, it shouod be run in the top (and only) project.  majorShrink altogether saves about 5,620k in Squeak 2.4"

	Smalltalk discardVMConstruction.  "666k"
	Smalltalk discardSoundSynthesis.  "358k"
	Smalltalk discardOddsAndEnds.  "158k"
	Smalltalk discardNetworking.  "243k"
	Smalltalk discardMorphic.  "2,494k"
	Symbol rehash.  "40k"
	"Above by itself saves about 3,960k"

"
	| a | a _ Smalltalk garbageCollect.
	Smalltalk majorShrink.
	Smalltalk garbageCollect - a
"
	"Remove references to a few classes to be deleted, so that they won't leave obsolete versions around."
	FileList removeSelector: #fileIntoNewChangeSet.
	ChangeSet class compile: 'defaultName
		^ ''Changes'' ' classified: 'initialization'.
	ScreenController removeSelector: #openChangeManager.
	ScreenController removeSelector: #exitProject.
	ScreenController removeSelector: #openProject.
	ScreenController removeSelector: #viewImageImports.

	"Now delete lots of classes.."
	SystemOrganization removeSystemCategory: 'Graphics-Symbols'.
	SystemOrganization removeSystemCategory: 'Graphics-Files'.
	SystemOrganization removeSystemCategory: 'Interface-Projects'.
	SystemOrganization removeSystemCategory: 'System-Object Storage'.

	Smalltalk removeClassNamed: #FormSetFont.
	Smalltalk removeClassNamed: #FontSet.
	Smalltalk removeClassNamed: #InstructionPrinter.
	Smalltalk removeClassNamed: #ChangeSorter.
	Smalltalk removeClassNamed: #DualChangeSorter.
	Smalltalk removeClassNamed: #EmphasizedMenu.
	Smalltalk removeClassNamed: #MessageTally.

	StringHolder class removeSelector: #originalWorkspaceContents.
	CompiledMethod removeSelector: #symbolic.

	RemoteString removeSelector: #makeNewTextAttVersion.
	Utilities class removeSelector: #absorbUpdatesFromServer.
	Smalltalk removeClassNamed: #PenPointRecorder.
	Smalltalk removeClassNamed: #Path.
	Smalltalk removeClassNamed: #Base64MimeConverter.
	Smalltalk removeClassNamed: #EToySystem.
	Smalltalk removeClassNamed: #RWBinaryOrTextStream.
	Smalltalk removeClassNamed: #AttributedTextStream.

	TextStyle allInstancesDo:
		[:ts | ts newFontArray: (ts fontArray copyFrom: 1 to: (2 min: ts fontArray size))].
	ListParagraph initialize.
	PopUpMenu initialize.
	StandardSystemView initialize.

	Smalltalk noChanges.
	ChangeSorter classPool at: #AllChangeSets 
		put: (OrderedCollection with: Smalltalk changes).

	[Smalltalk removeAllUnSentMessages > 0]
		whileTrue:
		[Smalltalk unusedClasses do: [:c | (Smalltalk at: c) removeFromSystem]].
	Smalltalk allClassesDo: [:c | c zapOrganization].
	Symbol rehash.
! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'di 3/20/1999 13:22'!
removeAllUnSentMessages   "Smalltalk removeAllUnSentMessages"
	"[Smalltalk unusedClasses do: [:c | (Smalltalk at: c) removeFromSystem].
		Smalltalk removeAllUnSentMessages > 0] whileTrue." 

	"Remove all implementations of unsent messages."
	| sels n |
	sels _ self allUnSentMessages.

	"The following should be preserved for doIts, etc"
	#(browseAllSelect: printSpaceAnalysis lastRemoval
		scrollBarValue: scrollBarMenuButtonPressed: 
		withSelectionFrom:  to: removeClassNamed:
		dragon: hilberts: mandala: web test3 factorial benchmark benchFib
		newDepth: restoreAfter: forgetDoIts
		removeAllUnSentMessages abandonSources removeUnreferencedKeys
		reclaimDependents zapOrganization condenseChanges browseObsoleteReferences
		subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:
		methodsFor:stamp: methodsFor:stamp:prior: instanceVariableNames:
		startTimerInterruptWatcher unusedClasses) do:
		[:sel | sels remove: sel ifAbsent: []].
	"The following may be sent by perform: in dispatchOnChar..."
	(ParagraphEditor classPool at: #CmdActions) asSet do:
		[:sel | sels remove: sel ifAbsent: []].
	(ParagraphEditor classPool at: #ShiftCmdActions) asSet do:
		[:sel | sels remove: sel ifAbsent: []].
	sels size = 0 ifTrue: [^ 0].

	n _ 0. Smalltalk allBehaviorsDo: [:x | n _ n+1].
	'Removing ', sels size printString , ' messages . . .'
		displayProgressAt: Sensor cursorPoint
		from: 0 to: n
		during:
		[:bar |
		n _ 0.
		self allBehaviorsDo:
			[:class | bar value: (n _ n+1).
			sels do:
				[:sel | class removeSelectorSimply: sel]]].
	MethodDictionary allInstancesDo: [:d | d rehash].
	^ sels size! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'di 3/20/1999 13:21'!
unusedClasses
	"Warning: Slow!! Enumerates all classes in the system and returns a list of those that are apparently unused. A class is considered in use if it (a) has subclasses (b) has instances or (c) is referred to by some method. Obsolete classes are not included in this list."
	"Smalltalk unusedClasses"

	| unused c n |
	unused _ SortedCollection new.

'Scanning for unused classes...'
	displayProgressAt: Sensor cursorPoint
	from: 0 to: Metaclass instanceCount
	during: [:bar | n _ 0.

	Metaclass allInstancesDo: [:meta | bar value: (n _ n+1).
		c _ meta soleInstance.
		((c ~~ nil) and:
		 [('AnOb*' match: c name asString) not]) ifTrue: [
			((c subclasses size = 0) and:
			 [(c inheritsFrom: FileDirectory) not & (c instanceCount = 0) and:
			 [(Smalltalk allCallsOn: (Smalltalk associationAt: c name)) size = 0]])
				ifTrue: [unused add: c name]]]].

	^ unused asArray
! !


!Wonderland class methodsFor: 'instance creation' stamp: 'di 4/11/1999 12:15'!
new   "Wonderland new" 
	"Create and initialize a new Wonderland."

	(self confirm: 'Wonderland is a work in progress.
In the absence of 3D primitive display support,
it will probably be too slow to be interesting.
We suggest that you wait until the next
version of Squeak to try using it.
Do you want to try anyway?') ifFalse: [^ self].
	Display depth = 32 ifFalse:
		[(self confirm: 'I need the Display depth to be 32 bits.
Shall I do this now for you?') ifFalse: [^ self].
		Display newDepth: 32].
	^ super new initialize
! !


SystemDictionary removeSelector: #makeTenuredGarbageWithForwarding!
