'From Squeak3.6alpha of ''17 March 2003'' [latest update: #5278] on 27 June 2003 at 5:50:03 am'!
"Change Set:		SMCardUndeclaredFix-nk
Date:			26 June 2003
Author:			Ned Konz

This fixes a couple of problems in SqueakMap related to 3.6a removals and changes. It is intended to work in older images as well.

First, the SMCard interface to mail a comment to an author has been changed to use MailSender, Celeste, or AdHocComposition, as available.

Second, the SMLoader checks for the existence of a registered WebBrowser or Scamper before offering to open a browser in its help.

"!


!SMCard methodsFor: 'mail' stamp: 'nk 6/26/2003 23:01'!
mailUserName
	"Answer the mail user's name, but deal with some historical mail senders."

	| mailSender |
	mailSender _ (Smalltalk at: #MailSender ifAbsent: [ Smalltalk at: #Celeste ifAbsent: []]).
	^mailSender
		ifNil: [ FillInTheBlank request: 'What is your email address?' ]
		ifNotNil: [ mailSender userName ].
! !

!SMCard methodsFor: 'mail' stamp: 'nk 6/26/2003 23:19'!
sendMail: aString
	"Send the given mail message, but check for modern mail senders."

	| server |

	Smalltalk at: #MailSender ifPresent: [ :mailSender |
		^mailSender sendMessage: ((Smalltalk at: #MailMessage) from: aString).
	].

	Smalltalk at: #MailComposition ifPresent: [ :mailComposition |
		^mailComposition new
			messageText:  aString;
			open
	].
	
	Smalltalk at: #Celeste ifPresent: [ :celeste |
		celeste isSmtpServerSet ifTrue: [
			Smalltalk at: #CelesteComposition ifPresent: [ :celesteComposition |
				^celesteComposition
					openForCeleste: celeste current 
					initialText: aString
			]
		]
	].

	Smalltalk at: #AdHocComposition ifPresent: [ :adHocComposition |
		server _  FillInTheBlank request: 'What is your mail server for outgoing mail?'.
		^adHocComposition 
			openForCeleste: server
			initialText: aString
	].

	^self inform: 'Sorry, no known way to send the message'.
	 	! !

!SMCard methodsFor: 'mail' stamp: 'nk 6/27/2003 05:38'!
sendMailTo: recipient
	"Send mail to the given recipient. Try to use the first of:
	- MailSender (with its registered composition class)
	- Celeste
	- AdHocComposition
	for compatibility with 3.5 and 3.6 images"
	| userName messageStream versionString |

	versionString _ currentVersion.
	versionString isEmptyOrNil ifTrue: [ versionString _ self smartVersion ].

	userName _ self mailUserName.

	messageStream _ WriteStream on: (String new: 1500).
	messageStream nextPutAll: 'From: '; nextPutAll: userName; cr;
		nextPutAll: 'To: '; nextPutAll: recipient; cr;
		nextPutAll: 'Subject: '; nextPutAll: self name; space; nextPutAll: versionString; cr.

	self sendMail: messageStream contents! !


!SMLoader methodsFor: 'gui building' stamp: 'nk 6/27/2003 05:42'!
help
	"Present help text. If there is a web server available, offer to open it.
	Use the WebBrowser registry if possible, or Scamper if available."
	| message browserClass |
	message _ 'Welcome to the SqueakMap package loader. 
The names of packages are followed by (installed version -> latest version). 
If there is no arrow, your installed version of the package is the latest.
The checkbox menu items at the bottom let you modify which packages 
you''ll see. Take a look at them - only some packages are shown initially. 
The options available for a package depend on how it was packaged. 
If you like a package or have comments on it, please contact
the author or the squeak mailing list.'.

	browserClass _ Smalltalk at: #WebBrowser ifPresent: [ :registry | registry default ].
	browserClass _ browserClass ifNil: [ Smalltalk at: #Scamper ifAbsent: [ ^self inform: message ]].

	(self confirm: message, '
Would you like to view more detailed help on the SqueakMap swiki page?') 
	ifTrue: [ browserClass openOnUrl: 'http://minnow.cc.gatech.edu/squeak/2726' asUrl]! !

