'From Squeak3.8alpha of ''17 July 2004'' [latest update: #5976] on 18 July 2004 at 4:00:51 pm'!
"Change Set:		TTCFontSizeRepair-nk
Date:			18 July 2004
Author:			Ned Konz

The changes to the TTCFonts done in CS 5923PSPointSizeFix-nk didn't clear out the derivative fonts cache.

This left some fonts with a discrepancy between the size of the regular and bold derivatives.

Where this is noticeable is where you had a default font (perhaps the window title font or the etoys font) set to a TTCFont (like ComicSansMS).

This CS forces the TTCFonts with bad derivative sizes to rebuild themselves.

This fixes a problem with the distribution image and also makes the old windows look better. It cures the problem that we've seen with bad flap and window title bar layout with some TTCFonts.

Then it does what it can to update all the users of those fonts.

As a side effect, it updates the buttons of old windows like the ones in the distribution image.

It also fixes AbstractFont>>textStyle to always answer a TextStyle, even if we have to create a new one. This avoids problems with some users of #textStyle.

"!


!AbstractFont methodsFor: 'accessing' stamp: 'nk 7/11/2004 21:15'!
textStyle
	^ TextStyle actualTextStyles detect:
		[:aStyle | aStyle fontArray includes: self] ifNone: [ TextStyle fontArray: { self } ]! !


!Preferences class methodsFor: 'fonts' stamp: 'nk 7/18/2004 15:34'!
refreshFontSettings
	"Try to update all the current font settings to make things consistent."

	self setFlapsFontTo: (self standardFlapFont);
		setEToysFontTo: (self standardEToysFont);
		setWindowTitleFontTo: (self windowTitleFont);
		setListFontTo: (self standardListFont);
		setMenuFontTo: (self standardMenuFont);
		setSystemFontTo: (TextStyle defaultFont);
		setCodeFontTo: (self standardCodeFont);
		setBalloonHelpFontTo: (BalloonMorph balloonFont).

	SystemWindow allSubInstancesDo: [ :s | | rawLabel |
		rawLabel := s getRawLabel.
		rawLabel owner vResizing: #spaceFill.
		rawLabel font: rawLabel font.
		s setLabel: s label.
		s replaceBoxes ].! !


!StandardSystemView class methodsFor: 'class initialization' stamp: 'nk 7/11/2004 21:11'!
setLabelStyle
	| aFont |
	"StandardSystemView setLabelStyle"
	aFont _ Preferences windowTitleFont.
	LabelStyle _ TextStyle fontArray: { aFont }.
	LabelStyle gridForFont: 1 withLead: 0! !


!SystemWindow methodsFor: 'initialization' stamp: 'nk 7/11/2004 22:28'!
replaceBoxes
	"Rebuild the various boxes."
	self setLabelWidgetAllowance.
	closeBox ifNotNilDo: [ :m | m delete. self addCloseBox. ].
	expandBox ifNotNilDo: [ :m | m delete. self addExpandBox. ].
	menuBox ifNotNilDo: [ :m | m delete. self addMenuControl. ].
	collapseBox ifNotNilDo: [ :m | m delete. labelArea addMorph: (collapseBox := self createCollapseBox) ].
	self setFramesForLabelArea.
	self setWindowColor: self paneColor ! !

!SystemWindow methodsFor: 'panes' stamp: 'nk 7/11/2004 21:54'!
updateBox: anIconMorph color: aColor 
	| fill |
	anIconMorph isNil
		ifTrue: [^ self].
	anIconMorph
		extent: self boxExtent;
		useRoundedCorners.
	fill := GradientFillStyle ramp: {0.0 -> aColor muchLighter muchLighter. 1.0 -> aColor twiceDarker}.
	
	fill origin: anIconMorph topLeft + (5 @ 5).
	fill direction: anIconMorph extent.

	anIconMorph fillStyle: fill.
	anIconMorph borderWidth: ((Preferences alternativeWindowLook
					and: [Preferences alternativeWindowBoxesLook])
				ifTrue: [1]
				ifFalse: [0]);
		borderColor: aColor darker! !


!TTCFont methodsFor: 'accessing' stamp: 'nk 7/18/2004 15:32'!
pointSize: aNumber

	self privatePointSize: aNumber.
	derivatives ifNotNil: [ derivatives do: [ :f | f ifNotNil: [ f privatePointSize: aNumber ]]].
! !

!TTCFont methodsFor: 'accessing' stamp: 'nk 7/18/2004 15:31'!
privatePointSize: aNumber 
	pointSize = aNumber
		ifFalse: [pointSize := aNumber.
			self flushCache]! !


!TTCFont class methodsFor: 'other' stamp: 'nk 7/18/2004 15:35'!
repairDerivativeFonts
	"Fix the cases where the derivatives are a different size than the originals."

	"
	TTCFont repairDerivativeFonts.
	"
	self allInstancesDo: [ :font | font pointSize: font pointSize ].
	Preferences refreshFontSettings.! !

"Postscript:
do the real fixups."

[ TTCFont repairDerivativeFonts ]
	ensure: [ Cursor wait showWhile: [ Preferences refreshFontSettings ]].!

