'From Squeak3.9alpha of ''2 November 2004'' [latest update: #6533] on 15 December 2004 at 12:03:58 am'!
"Change Set:		dupAllCtrlAltKeysPref-dew
Date:			13 December 2004
Author:			Doug Way

Adds a new preference duplicateAllCtrlAndAltkeys to duplicate all Ctrl and Alt modifier key commands, which causes all Ctrl key commands to behave the same as Alt-key commands.  The original Ctrl (a.k.a. 'shift') key commands are still available via Ctrl-Shift and Alt-Shift.

This differs from the existing ctrlAlt prefs which only duplicate/swap the eight editing keys.  Also includes code to prevent these other two conflicting prefs from being activated at the same time.

The three related prefs (swapCtrlAndAltKeys, duplicateCtrlAndAltKeys, duplicateAllCtrlAndAltKeys) should really be made into a radio button preferences set sometime in the future (like the Halo look preferences), since only one can be turned on at a time.  This would eliminate the need for the #changed methods.

See 'modifier' discussion on squeak-dev from 12/3/2004 for more details on why this was provided."!


!InputSensor class methodsFor: 'class initialization' stamp: 'dew 12/13/2004 23:21'!
installKeyDecodeTable
	"Create a decode table that swaps some keys if 
	Preferences swapControlAndAltKeys is set"
	KeyDecodeTable _ Dictionary new.
	Preferences duplicateControlAndAltKeys 
		ifTrue: [ self defaultCrossPlatformKeys do:
				[ :c | self installDuplicateKeyEntryFor: c ] ].
	Preferences swapControlAndAltKeys 
		ifTrue: [ self defaultCrossPlatformKeys do:
				[ :c | self installSwappedKeyEntryFor: c ] ].
	Preferences duplicateAllControlAndAltKeys
		ifTrue: [ (Character allCharacters select: [:ea | ea isAlphaNumeric]) do:
				[ :c | self installDuplicateKeyEntryFor: c ] ].
! !

!InputSensor class methodsFor: 'preference change notification' stamp: 'dew 12/14/2004 23:54'!
duplicateAllControlAndAltKeysChanged
	"The Preference for duplicateAllControlAndAltKeys has changed; reset the other two."
	"At some point the various exclusive CtrlAlt-key prefs should become a radio button set, then these methods wouldn't be needed."
	(Preferences
		valueOfFlag: #swapControlAndAltKeys
		ifAbsent: [false]) ifTrue: [
			self inform: 'Resetting swapControlAndAltKeys preference'.
			(Preferences preferenceAt: #swapControlAndAltKeys) rawValue: false.
		].
	(Preferences
		valueOfFlag: #duplicateControlAndAltKeys
		ifAbsent: [false]) ifTrue: [
			self inform: 'Resetting duplicateControlAndAltKeys preference'.
			(Preferences preferenceAt: #duplicateControlAndAltKeys) rawValue: false.
		].
	self installKeyDecodeTable.
! !

!InputSensor class methodsFor: 'preference change notification' stamp: 'dew 12/13/2004 18:58'!
duplicateControlAndAltKeysChanged
	"The Preference for duplicateControlAndAltKeys has changed; reset the other two."
	(Preferences
		valueOfFlag: #swapControlAndAltKeys
		ifAbsent: [false]) ifTrue: [
			self inform: 'Resetting swapControlAndAltKeys preference'.
			(Preferences preferenceAt: #swapControlAndAltKeys) rawValue: false.
		].
	(Preferences
		valueOfFlag: #duplicateAllControlAndAltKeys
		ifAbsent: [false]) ifTrue: [
			self inform: 'Resetting duplicateAllControlAndAltKeys preference'.
			(Preferences preferenceAt: #duplicateAllControlAndAltKeys) rawValue: false.
		].
	self installKeyDecodeTable.
! !

!InputSensor class methodsFor: 'preference change notification' stamp: 'dew 12/13/2004 18:58'!
swapControlAndAltKeysChanged
	"The Preference for swapControlAndAltKeys has changed; reset the other two."
	(Preferences
		valueOfFlag: #duplicateControlAndAltKeys
		ifAbsent: [false]) ifTrue: [
			self inform: 'Resetting duplicateControlAndAltKeys preference'.
			(Preferences preferenceAt: #duplicateControlAndAltKeys) rawValue: false.
		].
	(Preferences
		valueOfFlag: #duplicateAllControlAndAltKeys
		ifAbsent: [false]) ifTrue: [
			self inform: 'Resetting duplicateAllControlAndAltKeys preference'.
			(Preferences preferenceAt: #duplicateAllControlAndAltKeys) rawValue: false.
		].
	self installKeyDecodeTable.
! !


!Preferences class methodsFor: 'standard queries'!
duplicateAllControlAndAltKeys
	^ self
		valueOfFlag: #duplicateAllControlAndAltKeys
		ifAbsent: [true]! !

"Postscript:
Add the #duplicateAllControlAndAltKeys preference and set it to true.  Also re-add some existing preferences just to move them into a new #keyboard category, and tweak the balloon help.  Then set up the #changed notification for the ctrlAlt prefs."

Preferences addPreference: #duplicateAllControlAndAltKeys category: #keyboard default: true balloonHelp: 'If true, duplicates all alt- keys as ctrl-keys (making ctrl-c be copy as well as alt-c).
Cannot be true if swapControlAndAltKeys or duplicateControlAndAltKeys are true.'.

Preferences addPreference: #duplicateControlAndAltKeys category: #keyboard default: false balloonHelp: 'If true, duplicates some alt- keys as ctrl- keys (making ctrl-c be copy as well as alt-c).
Cannot be true if swapControlAndAltKeys or duplicateAllControlAndAltKeys are true.'.

Preferences addPreference: #swapControlAndAltKeys category: #keyboard default: false balloonHelp:  'If true, swaps some control- and alt-keys (making ctrl-c be copy instead of alt-c). 
Cannot be true if duplicateControlAndAltKeys or duplicateAllControlAndAltKeys is true.'.

(Preferences preferenceAt: #honorDesktopCmdKeys) categoryList: #(#keyboard #menus).

(Preferences preferenceAt: #duplicateAllControlAndAltKeys) rawValue: false.
(Preferences preferenceAt: #duplicateControlAndAltKeys) rawValue: false.
(Preferences preferenceAt: #swapControlAndAltKeys) rawValue: false.

(Preferences preferenceAt: #duplicateAllControlAndAltKeys) changeInformee: InputSensor changeSelector: #duplicateAllControlAndAltKeysChanged.
(Preferences preferenceAt: #duplicateControlAndAltKeys) changeInformee: InputSensor changeSelector: #duplicateControlAndAltKeysChanged.
(Preferences preferenceAt: #swapControlAndAltKeys) changeInformee: InputSensor changeSelector: #swapControlAndAltKeysChanged.

(Preferences preferenceAt: #duplicateAllControlAndAltKeys) preferenceValue: true.
!

